
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Read_PS2_VHDL is
    Port ( Clk :      in  STD_LOGIC;
           PS2_Clk :  in  STD_LOGIC;
           PS2_Data : in  STD_LOGIC;
           Reset :    in  STD_LOGIC;
           PS2_Code : inout  STD_LOGIC_VECTOR (7 downto 0);
           Left_dir : inout  STD_LOGIC_VECTOR (1 downto 0);
           Right_dir :inout  STD_LOGIC_VECTOR (1 downto 0);
           Serve :    inout  STD_LOGIC);
end Read_PS2_VHDL;

architecture Behavioral of Read_PS2_VHDL is
 	signal read:       std_logic;   
	signal Data_Ready: std_logic;
	signal stopkey: 	 std_logic;
	signal trigger: 	 std_logic;
	
	COMPONENT ps2_ctrl_VHDL
	PORT(	Clk : IN std_logic;
		   Reset : IN std_logic;
		   PS2_Clk : IN std_logic;
		   PS2_Data : IN std_logic;
		   DoRead : IN std_logic;    
		   Trigger : INOUT std_logic;      
		   Scan_Code : OUT std_logic_vector(7 downto 0);
		   Scan_ready : OUT std_logic	);
	END COMPONENT;
begin

	Inst_ps2_ctrl_VHDL: ps2_ctrl_VHDL PORT MAP(
		Clk =>        Clk ,
		Reset =>      Reset,
		PS2_Clk =>    PS2_Clk,
		PS2_Data =>   PS2_Data,
		Scan_Code =>  PS2_Code,
		Scan_ready => Data_Ready,
		Trigger =>    Trigger,
		DoRead =>     Read);
		
	This_process_recieves_and_decodes_scan_codes_from_the_keyboard:
   process( Clk)
	   type     Key_states is (Not_Pressed, Pressed);
		variable Key_state: Key_states;
	begin
	   if rising_edge( Clk) then
			if Reset='1' then
				right_dir <= "00";						  -- No right paddle movement
				left_dir  <= "00";						  -- No left paddle movement
				serve     <=  '1';						  -- Start with ball being served
				read      <=  '1';						  -- Ready to receive scan code
				stopkey   <=  '0';						  -- Stop key code has not been read
				Key_state :=  Not_Pressed;	
				
			elsif (Data_Ready='1') or (Read='0') then
				case Key_state is
					when Not_Pressed =>
						if Stopkey='0' then
							case (ps2_code) is
								when "01110101" =>	  				   -- up arrow key
										right_dir <= "01";	  			-- right up
								when "01110010" =>						-- down arrow key
										 right_dir <= "10";	  			-- right down
								when "00011101" =>					   -- w key	
										 left_dir <= "01";				--	left up
								when "00011011" =>						-- s key
										 left_dir <= "10";				-- left down
								when "00101001" =>						-- space bar key
										 serve <= '1';						-- serve ball
								when "11110000" =>						-- A key has been released
										stopkey <= '1';	  				-- set stopkey bit 
										serve   <= '0';
								when others =>									
							end case;  		   	  -- end of case (ps2_code) statement
							Key_state := Pressed;
						else  	 				  -- new data is telling which key was just released
							stopkey <= '0';
							case (ps2_code) is
								when "01110101" =>	  				   -- up arrow key
									 if (right_dir = "01") then
										right_dir <= "00";				-- stop right paddle up	motion
									 end if;
								when "01110010" =>						-- down arrow key
									 if (right_dir = "10") then   
										 right_dir <= "00";	  			-- stop right paddle down motion
									 end if;
								when "00011101" =>					   -- w key	
									 if (left_dir = "01") then  
										 left_dir <= "00";				--	stop left paddle up motion
									 end if;
								when "00011011" =>						-- s key
									 if (left_dir = "10") then   
										 left_dir <= "00";				-- stop left paddle down motion
									 end if;
								when "11100000" =>						-- Extended code key
										 stopkey <= '1';			   	-- Check next ps2 entry for stop											 
								when "00101001" =>						-- space bar key
										 serve   <= '0';					-- stop serve ball
								when others => 
										 stopkey <= '0';								
							end case; 									      -- end of case (ps2_code) statement
							Key_state := Pressed;
						end if; 
						read <= '0';
						
					When Pressed =>
						if (Trigger='0') then 
							read <= '1';								-- resets read bit to enable PS2_Ctrl input
							Key_state := Not_Pressed;
						end if;
				end case;
		   end if;
      end if;		  
   end process This_process_recieves_and_decodes_scan_codes_from_the_keyboard;
end Behavioral;

