------------------------------------------------------------------
-- Copyright (c) 1995-2005 Xilinx, Inc.
-- All Right Reserved.
------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 8.1i
--  \   \         Application : 
--  /   /         Filename : vga_interface.vhd
-- /___/   /\     Timestamp : 9/20/2005 11:15:08
-- \   \  /  \ 
--  \___\/\___\ 
--
-- 
--   This Entity interprets the color output to the VGA monitor

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity vga_int is Port ( 
	CLK :			in  std_logic;
	COLOR: 		in  std_logic_vector ( 1 downto 0 );
	VSYNCH_IN: 	in  std_logic;
	HSYNCH_IN: 	in  std_logic;
	RED:			out std_logic;
	BLUE:			out std_logic;
	GREEN:		out std_logic;
	VSYNCH_OUT:	out std_logic;
	HSYNCH_OUT:	out std_logic);
end vga_int;

architecture behavioral of vga_int is
	signal VSYNCH_PIPE: std_logic;
	signal HSYNCH_PIPE: std_logic;
	
	signal   RGB:      Std_logic_vector( 2 downto 0);
	constant xBLACK:   Std_logic_vector( 2 downto 0) := "000";
	constant xRED:     Std_logic_vector( 2 downto 0) := "100";
	constant xGREEN:   Std_logic_vector( 2 downto 0) := "010";	
	constant xBLUE:    Std_logic_vector( 2 downto 0) := "001";
	constant xWHITE:   Std_logic_vector( 2 downto 0) := "111";
	constant xMAGENTA: Std_logic_vector( 2 downto 0) := "101"; 
	constant xCYAN: 	 Std_logic_vector( 2 downto 0) := "011"; 
	constant xYELLOW:  Std_logic_vector( 2 downto 0) := "110"; 
begin
	PIPELINE: process	 (CLK)
	begin
		if ( CLK = '1' and CLK'event ) then
--			VSYNCH_OUT  <= VSYNCH_PIPE; HSYNCH_OUT  <= HSYNCH_PIPE;
--			HSYNCH_PIPE <= HSYNCH_IN; 	 VSYNCH_PIPE <= VSYNCH_IN;
			VSYNCH_OUT  <= VSYNCH_IN;
			HSYNCH_OUT  <= HSYNCH_IN;
		end if;
	end process;
	
	COLOR_LUT: process  (CLK)
	begin
		if ( CLK = '1' and CLK'event ) then
			RGB <= xBLACK;
			case COLOR is
					when "00" =>      -- Black (only ???)
						RGB <= xBLACK;
					when "01" =>     
						RGB <= xRED;
					when "10" =>       
						RGB <= xMAGENTA;
					when "11" =>      
						RGB <= xCYAN;
					when others => NULL;
				end case;						
		end if;
	end process;
   
	RED   <= RGB(2);
	GREEN <= RGB(1);
	BLUE  <= RGB(0);
end behavioral;